import string

from django.db.models import Q
from django.http import HttpResponse
from django.contrib.auth.hashers import make_password
from main.models import DefaultSettings, Vm, User, Iso, Request, RequestType
from main.UserProxMoxDTO import UserProxMoxDTO
from main.VmProxMoxDTO import VmProxMoxDTO
from datetime import datetime, timedelta, timezone


def get_default_settings() -> dict:
    queryset = DefaultSettings.objects.all()
    output_settings_dict = {}
    for line in queryset:
        output_settings_dict['lease_time_months'] = line.lease_time_months
        output_settings_dict['memory_mbs'] = line.memory_mbs
        output_settings_dict['storage_gbs'] = line.storage_gbs
        output_settings_dict['cores'] = line.cores
        output_settings_dict['amount_of_vms'] = line.amount_of_vms
        output_settings_dict['notification_email'] = line.notification_email
    return output_settings_dict


def update_default_settings(lease_time: int, memory: int, storage: int, cores: int, amount_of_vms: int, notification_email: str) -> HttpResponse:
    ds = DefaultSettings.objects.get(id=1)
    ds.lease_time_months = lease_time
    ds.memory_mbs = memory
    ds.storage_gbs = storage
    ds.cores = cores
    ds.amount_of_vms = amount_of_vms
    ds.notification_email = notification_email
    ds.save()
    return HttpResponse("Updated successfully!")


# Returns id of last inserted VM
def insert_vm(vm: VmProxMoxDTO) -> int:
    vm_model = Vm()
    vm_model.name = vm.vm_name
    vm_model.memory_mbs = vm.vm_memory
    vm_model.storage_gbs = int(vm.vm_disk_size[:-1])
    vm_model.cores = vm.vm_cores
    vm_model.end_date = datetime.now() + timedelta(days=vm.vm_expire_days)
    results = Iso.objects.filter(volid=vm.vm_iso_file)
    if results.exists():
        vm_model.iso_id = results[0]
    else:
        vm_model.iso_id = None
    # vm_model.member_email
    vm_model.save()

    return vm_model.id


def insert_vm_with_id(vm: VmProxMoxDTO, vmid: int) -> HttpResponse:
    vm_model = Vm()
    vm_model.name = vm.vm_name
    vm_model.memory_mbs = vm.vm_memory
    vm_model.storage_gbs = int(vm.vm_disk_size[:-1])
    vm_model.cores = vm.vm_cores
    vm_model.end_date = datetime.now() + timedelta(days=vm.vm_expire_days)
    vm_model.iso_id = None
    try:
        vm_model.id = vmid
    except:
        pass
    vm_model.save()
    return HttpResponse("Vm Inserted")


def add_user_to_vm(user_id: int, vm_id: int) -> HttpResponse:
    try:
        vm_model = Vm.objects.get(id=vm_id)
    except Vm.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")

    # If record of VM exists update it
    try:
        user = User.objects.get(user_id=user_id)
        vm_model.user_id = user
        vm_model.save()
    except User.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")

    return HttpResponse("Record updated!")


def get_list_iso_from_db() -> tuple:
    queryset = Iso.objects.all()
    return queryset


def is_iso_in_sql_db(volidinp: str) -> bool:
    queryset = Iso.objects.filter(volid=volidinp)
    # If query set empty (no matches found) return false
    if queryset:
        return True
    else:
        return False


def is_user_in_sql_db(username: str) -> bool:
    queryset = User.objects.filter(username=username)
    # If query set empty (no matches found) return false
    if queryset:
        return True
    else:
        return False


def is_vm_in_sql_db(vmid: int) -> bool:
    queryset = Vm.objects.filter(id=vmid)
    # If query set empty (no matches found) return false
    if queryset:
        return True
    else:
        return False


def insert_iso(os_name: str, volidinp: str) -> HttpResponse:
    iso_model = Iso()
    iso_model.os_name = os_name
    iso_model.volid = volidinp
    iso_model.is_visible = False
    iso_model.save()
    return HttpResponse("Data inserted!")


def remove_iso(id: int) -> HttpResponse:
    try:
        iso_model = Iso.objects.get(id=id)
        iso_model.delete()
    except Iso.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")
    return HttpResponse("Data removed!")


# Returns a dictionary of volid - os_name of records, where is_visible = True
def get_list_visible_iso_from_db() -> dict:
    results = get_list_iso_from_db()
    output_iso = {}
    for record in results:
        if record.is_visible is True:
            output_iso[record.volid] = record.os_name
    return output_iso


def change_iso_visibility(id: int) -> HttpResponse:
    try:
        iso_model = Iso.objects.get(id=id)
        # Change is_visible field to the opposite
        if iso_model.is_visible == 1:
            iso_model.is_visible = 0
            iso_model.save()
        else:
            iso_model.is_visible = 1
            iso_model.save()
    except Iso.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")
    return HttpResponse("Visbility Changed!")


def change_user_visibility(username: string) -> HttpResponse:
    try:
        user = User.objects.get(username=username)
        if user.is_active == 1:
            user.is_active = 0
            user.save()
        else:
            user.is_active = 1
            user.save()
    except User.DoesNotExist:
        return HttpResponse("User not found")
    return HttpResponse("Activity changed")


# Returns last inserted user's id
def insert_member(userdto: UserProxMoxDTO) -> int:
    limits = get_default_settings()
    user_model = User()
    user_model.email = userdto.email
    user_model.first_name = userdto.firstname
    user_model.last_name = userdto.lastname
    user_model.password = make_password(userdto.password)
    user_model.username = userdto.username
    user_model.maxAmountOfVms = int(limits['amount_of_vms'])
    user_model.save()
    assign_vlan_tag(user_model.user_id)

    return user_model.user_id


def assign_vlan_tag(userid: int) -> int:
    try:
        user = User.objects.get(user_id=userid)
        user.vlanTag = userid
        user.save()
    except User.DoesNotExist:
        return None
    return user.vlanTag


def get_list_users_from_db() -> tuple:
    queryset = User.objects.all()
    return queryset


def get_list_unapproved_users() -> tuple:
    queryset = Iso.objects.filter(is_active=0)
    return queryset


def is_username_unique(username: str) -> bool:
    try:
        user_model = User.objects.get(username=username)
        if user_model is not None:
            return False
        else:
            return True
    except User.DoesNotExist:
        return True


def get_user_vm_count(user_inp: User) -> int:
    queryset = Vm.objects.filter(user_id=user_inp)
    return queryset.count()


def edit_iso_db(iso_id: int, os_name_inp: str) -> HttpResponse:
    try:
        iso_model = Iso.objects.get(id=iso_id)
        iso_model.os_name = os_name_inp
        iso_model.save()
    except Iso.DoesNotExist:
        return HttpResponse("Object not found.")
    return HttpResponse("Changes saved!")


def get_iso_by_id(iso_id: int) -> dict:
    result = Iso.objects.get(id=iso_id)
    iso_dict = {
        'os_name': result.os_name,
        'iso_id': result.id,
        'volid': result.volid,
        'is_visible': result.is_visible
    }
    return iso_dict


def get_user_info_from_vmid(vmid: int):
    try:
        # Connect to the database and retrieve the VM object using vmid
        vm = Vm.objects.get(id=vmid)
        # Retrieve user_id and user_name from the VM object
        user_temp = vm.user_id
        user_name = str(user_temp.first_name) + " " + str(user_temp.last_name)
        end_date = vm.end_date

        return user_temp.user_id, user_name, end_date

    except Vm.DoesNotExist:
        # Handle the case when VM is not found in the database
        return None, None, None

    except Exception as e:
        # Handle any other exceptions that may occur
        print(f"An error occurred: {e}")
        return None, None, None


def remove_vm_db(vmid: int):
    try:
        vm_model = Vm.objects.get(id=vmid)
        vm_model.delete()
    except Iso.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")
    return HttpResponse("Data removed!")


def get_vm_iso(vmid: int) -> dict:
    try:
        iso_model = Vm.objects.get(id=vmid).iso_id
        iso_dict = {
            'name': iso_model.os_name,
            'id': iso_model.id,
            'volid': iso_model.volid,
            'visible': iso_model.is_visible
        }
        return iso_dict
    except (Vm.DoesNotExist, Iso.DoesNotExist, AttributeError):
        iso_dict = {
            'name': None,
            'id': None,
            'volid': None,
            'visible': None
        }
        return iso_dict


def remove_user_db(username: str) -> HttpResponse:
    try:
        user_model = User.objects.get(username=username)
        user_model.delete()
    except User.DoesNotExist:
        # Handle the case when the object does not exist
        return HttpResponse("Object not found.")
    return HttpResponse("Data removed!")


def get_list_db_vm():
    queryset = Vm.objects.all()
    return queryset


def search_db_vm(keyword_search: str):
    queryset = Vm.objects.filter(
        Q(id__contains=keyword_search) |
        Q(name__contains=keyword_search) |
        Q(end_date__contains=keyword_search) |
        Q(user_id__first_name__contains=keyword_search) |
        Q(user_id__email__contains=keyword_search) |
        Q(user_id__last_name__contains=keyword_search) |
        Q(user_id__username__contains=keyword_search) |
        Q(iso_id__os_name__contains=keyword_search)
    )
    return queryset


def select_user_db(username: str) -> User:
    try:
        user_model = User.objects.get(username=username)
        return user_model
    except User.DoesNotExist:
        return None


def select_user_db_by_id(userid: int) -> User:
    try:
        user_model = User.objects.get(user_id=userid)
        return user_model
    except User.DoesNotExist:
        return None


def get_users_vms(username: str) -> list:
    try:
        user_model = User.objects.get(username=username)
    except User.DoesNotExist:
        return []
    queryset = Vm.objects.filter(user_id=user_model)
    return queryset


def select_vm_db(vmid: int) -> Vm:
    try:
        vm_model = Vm.objects.get(id=vmid)
        return vm_model
    except Vm.DoesNotExist:
        return None


def extend_vm_lease_time(vmid: int) -> HttpResponse:
    defaults = get_default_settings()
    try:
        vm_model = Vm.objects.get(id=vmid)
        vm_model.end_date = datetime.now() + timedelta(days=(defaults['lease_time_months'] * 365) / 12)
        vm_model.save()
        return HttpResponse("Date extended")
    except Vm.DoesNotExist:
        return HttpResponse("Object not found")


def insert_request_db(message: str, vm_id: int, user_id: int, requesttype: str) -> int:
    request = Request()
    request.message = message
    request.vm_id = select_vm_db(vm_id)
    request.user_id = User.objects.get(user_id=user_id)
    request.is_approved = None
    request.request_type_id = RequestType.objects.get(name=requesttype)
    request.save()
    return request.id


def get_request_types():
    return RequestType.objects.all()


def get_requests_by_user(user_id: int):
    return Request.objects.filter(user_id=user_id)


def get_all_requests():
    return Request.objects.all()


def get_requests_by_type(type_id: id):
    return Request.objects.filter(type_id=type_id)


def get_pending_requests():
    return Request.objects.filter(is_approved=None)


def get_approved_requests():
    return Request.objects.filter(is_approved=True)


def get_denied_requests():
    return Request.objects.filter(is_approved=False)


def approve_request(request_id: int):
    try:
        request = Request.objects.get(id=request_id)
        request.is_approved = True
        request.save()
    except Request.DoesNotExist:
        return
    return


def deny_request(request_id: int) -> HttpResponse:
    try:
        request = Request.objects.get(id=request_id)
        request.is_approved = False
        request.save()
    except Request.DoesNotExist:
        return HttpResponse("Not found")
    return HttpResponse("Updated")


def get_list_vms_to_expire():
    threshold_date = datetime.now() + timedelta(days=30)
    records = Vm.objects.filter(end_date__lte=threshold_date)
    print(records)
    return records


def get_request_by_id(requestid: int):
    try:
        model = Request.objects.get(id=requestid)
        return model
    except Request.DoesNotExist:
        return None


def get_list_vms_expired():
    today = datetime.now()
    queryset = Vm.objects.filter(end_date__lte=today)
    print(queryset)
    return queryset


def get_list_of_unconfirmed_users():
    four_hours_ago_time = datetime.now() - timedelta(hours=4)
    queryset = User.objects.filter(email_confirmed=0, date_joined__lt=four_hours_ago_time)
    return queryset


