from django.core.mail import send_mail, EmailMessage
from django.conf import settings
from main.db_functions import select_user_db, get_request_by_id, get_default_settings


def send_registration_approve(username: str, password: str):
    subject = 'Registration Approved - Proxmox Self Service'
    from_email = settings.DEFAULT_FROM_EMAIL
    user_email = str(select_user_db(username).email)
    recipient_list = [user_email, ]
    message = f'Welcome to Proxmox Self Service by NHL Stenden. You have just been approved to access the application. ' \
              f'You can use the same credentials to access Proxmox Self Service and Proxmox VE (make sure to login into PVE realm). ' \
              f'Username: {username} Password: {password}'
    send_mail(subject, message, from_email, recipient_list)


def send_email_confirmed_letter(username: str):
    subject = 'Email Confirmed - Proxmox Self Service'
    from_email = settings.DEFAULT_FROM_EMAIL
    user_email = str(select_user_db(username).email)
    recipient_list = [user_email, ]
    message = "Thank you for your request to register with Proxmox Self Service by NHL Stenden. Your email has been cofirmed!" \
              " Please wait until a member of staff grants you access to the application. As soon as you are approved, " \
              "you will receive an email with your credentials to access the web application"
    send_mail(subject, message, from_email, recipient_list)


def send_vm_expire_letter(email: str, vmname: str):
    subject = 'VM Expiring - Proxmox Self Service'
    from_email = settings.DEFAULT_FROM_EMAIL
    recipient_list = [email, ]
    message = f'Hello! Your VM {vmname} is about to expire! If you wish to continue using it, please log into your ' \
              f'Proxmox Self Help account and press the extend button next to the appropriate machine. The lease time will be' \
              f' extended automatically. If you do not take any action, your virtual machine will be removed  and any data ' \
              f'on it will be lost forever.'
    send_mail(subject, message, from_email, recipient_list)


def send_new_request_letter_admin(requestid: int):
    request_model = get_request_by_id(requestid)
    email = get_default_settings()['notification_email']
    type_req = str(request_model.request_type_id.name)
    name = str(request_model.user_id.first_name) + ' ' + str(request_model.user_id.last_name)
    subject = f'New request - {type_req}'
    from_email = settings.DEFAULT_FROM_EMAIL
    recipient_list = [email, ]
    message = f'There is a new request under {type_req} placed by user {name}. The message is: ' \
              f'{request_model.message} '
    send_mail(subject, message, from_email, recipient_list)


