from django.contrib.admin import display
from django.db import models
from django.contrib.auth.models import AbstractUser


# Create your models here.
# REQUIRED_FIELDS = ['id_email', 'first_name', 'last_name']


class User(AbstractUser):
    user_id = models.AutoField(primary_key=True, unique=True)
    email = models.EmailField(max_length=255)
    first_name = models.CharField(max_length=128, unique=False, null=True)
    last_name = models.CharField(max_length=128, unique=False, null=True)
    is_active = models.BooleanField(default=False)
    is_staff = models.BooleanField(default=False)
    maxAmountOfVms = models.IntegerField(null=True)
    is_superuser = models.BooleanField(default=False)
    password = models.CharField(max_length=255)
    vlanTag = models.IntegerField(null=True)
    last_login = models.DateTimeField(auto_now=True)
    date_joined = models.DateTimeField(auto_now_add=True)
    email_confirmed = models.BooleanField(default=False)


class DefaultSettings(models.Model):
    id = models.AutoField(primary_key=True, unique=True)
    lease_time_months = models.IntegerField()
    memory_mbs = models.IntegerField()
    storage_gbs = models.IntegerField()
    cores = models.IntegerField()
    notification_email = models.EmailField()
    amount_of_vms = models.IntegerField()


class Iso(models.Model):
    id = models.AutoField(primary_key=True, unique=True)
    is_visible = models.BooleanField()
    os_name = models.CharField(max_length=255)
    volid = models.CharField(max_length=320)


class Vm(models.Model):
    id = models.AutoField(primary_key=True, unique=True)
    name = models.CharField(max_length=64)
    user_id = models.ForeignKey(User, on_delete=models.CASCADE, null=True)
    end_date = models.DateTimeField()
    memory_mbs = models.IntegerField()
    storage_gbs = models.IntegerField()
    cores = models.IntegerField()
    iso_id = models.ForeignKey(Iso, on_delete=models.SET_NULL, max_length=11, null=True)


class RequestType(models.Model):
    id = models.AutoField(primary_key=True, unique=True)
    name = models.CharField(max_length=20, unique=True)


class Request(models.Model):
    id = models.AutoField(primary_key=True, unique=True)
    message = models.CharField(max_length=256)
    vm_id = models.ForeignKey(Vm, on_delete=models.CASCADE, max_length=11, null=True)
    request_type_id = models.ForeignKey(RequestType, on_delete=models.SET_NULL, max_length=11, null=True)
    is_approved = models.BooleanField(null=True, default=None)
    date_time = models.DateTimeField(auto_now=True)
    user_id = models.ForeignKey(User, on_delete=models.CASCADE)
