import time
from datetime import datetime, timezone
import proxmoxer
from django.contrib.auth.hashers import make_password
from django.contrib.sites.shortcuts import get_current_site
from django.core.mail import EmailMessage
from django.template.loader import render_to_string
from django.utils.encoding import force_bytes, force_str
from django.utils.http import urlsafe_base64_encode, urlsafe_base64_decode
from main.UserProxMoxDTO import UserProxMoxDTO
import os
from django.shortcuts import render
from django.urls import reverse
from django.contrib import messages
from main.Utils import create_vm, get_list_iso_urls, sanitize_vm_data_input, months_to_days, upload_iso, \
    sync_iso_from_ve_to_db, remove_iso_from_ve, create_user, is_email_nhl, change_user_enable, create_vm_from_template, \
    superuser_check, get_list_templates, add_user_to_vm_proxmox, get_resource_consumption, \
    remove_user_proxmox_ve, \
    get_list_all_users, get_list_vms_proxmox, sync_vms_ve_and_db, count_words, bytes_to_gb, generate_password, \
    get_list_permissions_of_vm
from main.db_functions import get_default_settings, insert_vm, insert_iso, get_list_visible_iso_from_db, remove_iso, \
    change_iso_visibility, insert_member, is_username_unique, change_user_visibility, add_user_to_vm, get_user_vm_count, \
    edit_iso_db, get_iso_by_id, update_default_settings, get_user_info_from_vmid, remove_vm_db, get_vm_iso, \
    remove_user_db, select_user_db, get_list_db_vm, get_users_vms, select_vm_db, extend_vm_lease_time, \
    insert_request_db, get_request_types, get_requests_by_user, get_all_requests, deny_request, approve_request, \
    search_db_vm, get_approved_requests, get_denied_requests, get_pending_requests, search_db_vm, select_user_db_by_id
from main import default_vm_node, default_storage, default_iso_storage, default_iso_path, ignore_vms_ids
from django.http import HttpResponse, HttpResponseRedirect
from main.VmProxMoxDTO import VmProxMoxDTO
from django.shortcuts import redirect
from main.mail_utils import send_registration_approve, send_email_confirmed_letter, send_new_request_letter_admin
from main.models import Iso, User
from proxmox.local_settings import MEDIA_ROOT
from django.contrib.auth.decorators import login_required, user_passes_test
from proxmox.settings import LOGIN_REDIRECT_URL
from main import proxmoxAuth
from .tokens import account_activation_token
import tasks


@login_required
def create_vm_view(request):
    # Get a list of limits from mysql DB
    limits = get_default_settings()
    user_logged = request.user  # Logged in user info (Django model)
    user_vm_count = get_user_vm_count(user_logged)
    # When user submits the form
    if request.POST:
        vm_name = request.POST.get('vm_name') + 'User' + user_logged.username  # Name for VM - VmNameUserUserName
        vm_cores = int(request.POST.get('vm_cores'))  # Number of CPU cores
        vm_memory = int(request.POST.get('vm_memory'))  # Memory size in MB
        vm_disk_size = int(request.POST.get('disk_size'))  # Disk size for the VM
        iso_file = str(request.POST.get('iso'))  # Path to the ISO file
        vmTempDTO = VmProxMoxDTO(vm_name, vm_cores, vm_memory, vm_disk_size,
                                 months_to_days(int(limits['lease_time_months'])), iso_file)
        vm_sanitized = sanitize_vm_data_input(vmTempDTO)

        # Check user's VM count
        if user_logged.is_superuser == 0 and user_vm_count >= int(user_logged.maxAmountOfVms):
            # Show an alert message
            messages.info(request, "You have exceeded your limit of available VMs")
            return render(request, "main/create.html")
        try:
            last_vm_id = insert_vm(vm_sanitized)
            info = create_vm(vm_sanitized, last_vm_id, user_logged.vlanTag)
            add_user_to_vm(user_logged.user_id, last_vm_id)  # Assign user in db
            user_response = add_user_to_vm_proxmox(user_logged.username, last_vm_id)

            # Redirect user to a different page if vm created
            return HttpResponseRedirect(reverse('vms_user'))
        except Exception as error:
            return HttpResponse(error)

    # Get a list of all ISO files from proxmox
    results = get_list_visible_iso_from_db()

    context = {
        "iso_options": results,
        "disk_size": limits['storage_gbs'],
        "memory": limits['memory_mbs'],
        "cores": limits['cores'],
        "lease": limits['lease_time_months'],
        "vm_count": user_vm_count
    }
    return render(request, "main/create.html", context)


def redirect_home(request):
    return HttpResponseRedirect(reverse('vms_user'))


@login_required
def index(request):
    return HttpResponseRedirect(reverse('vms_user'))


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def upload_iso_view(request):
    if request.method == 'POST':
        iso_file = request.FILES['iso_file']  # Get ISO file from web form
        os_name = request.POST.get('os_name')  # Get OS name
        upload_path = os.path.join(MEDIA_ROOT, iso_file.name)
        try:
            # Save the uploaded file to the upload path
            with open(upload_path, 'wb+') as destination:
                for chunk in iso_file.chunks():
                    destination.write(chunk)
            # Upload the ISO file to Proxmox VE
            volid = upload_iso(iso_file.name, upload_path)
        except Exception as error:
            return HttpResponse(error)
        # Save to sql db
        if len(os_name) == 0:
            os_name = volid
        insert_iso(os_name, volid)
        url = reverse('manage_iso')
        return HttpResponseRedirect(url)
    return render(request, 'admin/upload_iso.html')  # Render the upload form initially


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def dynamic_iso_table(request):
    entries = Iso.objects.all()
    return render(request, 'admin/manage_iso.html', {'entries': entries})


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def dynamic_user_table(request):
    # gets all the inactive users
    entries = User.objects.filter(is_active=0, email_confirmed=1)
    return render(request, 'admin/user_register_requests.html', {'entries': entries})


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def synchronize_iso_view(request):
    sync_iso_from_ve_to_db()
    url = reverse('manage_iso')
    return HttpResponseRedirect(url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def delete_iso(request, id=None):
    # Get data from Get request if its not empty
    iso_id = id
    if iso_id is not None:
        response_ve = remove_iso_from_ve(iso_id)  # Remove from Ve
        response_db = remove_iso(iso_id)  # Remove from DB
    url = reverse('manage_iso')
    return HttpResponseRedirect(url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def change_iso_visibility_view(request, id=None):
    # Get data from post
    iso_id = id
    if iso_id is not None:
        response = change_iso_visibility(iso_id)
    url = reverse('manage_iso')
    return HttpResponseRedirect(url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def edit_iso_view(request):
    iso_id = request.session.get('iso_edit_id')
    # Get ISO info from DB
    iso_dict = get_iso_by_id(iso_id)
    # When user submits form -> save changes and redirect back
    if request.method == 'POST':
        if iso_id is not None:
            os_name = str(request.POST.get('os_name'))
            if len(os_name) == 0:
                os_name = iso_dict['os_name']
            response = edit_iso_db(iso_id, os_name)
            return HttpResponseRedirect(reverse('manage_iso'))

    return render(request, 'admin/edit_iso.html', {'entries': iso_dict})


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def edit_iso_redirect_view(request, id=None):
    user = request.user
    if id is not None:
        request.session['iso_edit_id'] = id  # Save ISO id into session variable
        return HttpResponseRedirect(reverse('edit_iso'))

    # Go back if there is an issue with id in GET request
    url = reverse('manage_iso')
    return HttpResponseRedirect(url)


# Accept registration request
@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def change_user_activity(request, username=None):
    user_name = username
    if user_name is not None:
        # Make user active
        response = change_user_visibility(user_name)
        password_temp = generate_password()  # Generate password
        # Save user's new password in DB
        user_model = select_user_db(user_name)
        user_model.password = make_password(password_temp)
        user_model.save()
        # Create user in Proxmox
        userdto = UserProxMoxDTO(user_model.email, user_model.first_name, user_model.last_name, password_temp, user_name)
        response_create = create_user(userdto, user_model.user_id)  # Save to proxmox
        send_registration_approve(user_name, password_temp)
    url = reverse('manage_user')
    return HttpResponseRedirect(url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def deny_user(request, username=None):
    user_name = username
    if user_name is not None:
        response = remove_user_db(user_name)
        response_ve = remove_user_proxmox_ve(user_name)
    url = reverse('manage_user')
    return HttpResponseRedirect(url)


def create_user_view(request):
    # Check if user is loggd in
    if request.user.is_authenticated:
        # Redirect to a different page
        return HttpResponseRedirect(reverse('vms_user'))

    # When user submits form
    if request.POST:
        # Get all input info into a DTO
        email = request.POST.get('email')
        first_name = request.POST.get('fname')
        last_name = request.POST.get('lname')
        username = request.POST.get('username')

        # Check if email is from NHL
        if not is_email_nhl(email):
            messages.info(request, "Only @nhlstenden.com email address can be used")
            return render(request, "main/create_user.html")
        # Check if username is unique
        if not is_username_unique(username):
            messages.info(request, "Username already exists")
            return render(request, "main/create_user.html")

        userdto = UserProxMoxDTO(email, first_name, last_name, "qwerty", username)
        try:
            user_id = insert_member(userdto)  # Save to db
            user_model = select_user_db_by_id(user_id)
            activateEmail(request, user_model, email)  # Send email with email confirmation link
            messages.info(request, f'A letter has been sent to {email} with a confirmation link. Do not forget to check your spam folder!')
            return render(request, "main/create_user.html")
        except Exception as error:
            return HttpResponse(error)
    return render(request, "main/create_user.html")


@login_required
def templates_list_result(request):
    templates = get_list_templates()
    limits = get_default_settings()
    user_logged = request.user  # Logged in user info (Django model)
    user_vm_number = get_user_vm_count(user_logged)
    context = {
        "templates": templates,
        "disk_size": limits['storage_gbs'],
        "memory": limits['memory_mbs'],
        "cores": limits['cores'],
        "lease": limits['lease_time_months'],
        "vm_count": user_vm_number
    }
    if request.POST:
        # Check user's VM count
        if user_logged.is_superuser == 0 and user_vm_number >= int(limits['amount_of_vms']):
            # Show an alert message
            messages.info(request, "You have exceeded your limit of available VMs")
            return render(request, "main/create_template.html", context)
        template_id = request.POST.get('template')
        vm_name = request.POST.get('vm_name') + 'User' + user_logged.username  # Name for VM - VmNameUserUserName
        vm_cores = limits['cores']
        vm_memory = limits['memory_mbs']
        vm_disk_size = limits['storage_gbs']
        iso_file = "Template - " + str(template_id)
        vmTempDTO = VmProxMoxDTO(vm_name, vm_cores, vm_memory, vm_disk_size, months_to_days(int(limits['lease_time_months'])), iso_file)
        vm_sanitized = sanitize_vm_data_input(vmTempDTO)
        last_vm_id = insert_vm(vm_sanitized)  # Save to db
        info = create_vm_from_template(last_vm_id, vm_name, template_id, request.user.vlanTag)
        add_user_to_vm(user_logged.user_id, last_vm_id)  # Assign user in db
        user_response = add_user_to_vm_proxmox(user_logged.username, last_vm_id)

        # Redirect user to a different page if vm created
        return HttpResponseRedirect(reverse('vms_user'))

    return render(request, "main/create_template.html", context)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def default_settings_view(request):
    if request.POST:
        lease = int(request.POST.get('lease_time'))
        memory = int(request.POST.get("memory"))
        storage = int(request.POST.get("storage"))
        cores = int(request.POST.get("cores"))
        vm_amount = int(request.POST.get("amount_of_vms"))
        email = request.POST.get("notification_email")

        result = update_default_settings(lease, memory, storage, cores, vm_amount, email)

    limits = get_default_settings()
    context = {
        "disk_size": limits['storage_gbs'],
        "memory": limits['memory_mbs'],
        "cores": limits['cores'],
        "lease": limits['lease_time_months'],
        "vm_amount": limits['amount_of_vms'],
        "notification_email": limits['notification_email']
    }
    return render(request, 'admin/manage_default_settings.html', context)


@login_required
def requests_view(request):
    current_user = request.user
    if request.POST:
        message = request.POST.get("request")
        user_id = current_user.user_id
        # vm_id = int(request.POST.get("vm"))
        try:
            vm_id = int(request.POST.get("vm"))
        except:
            vm_id = None
        request_type = request.POST.get("request_type")
        lastid = insert_request_db(message=message, vm_id=vm_id, user_id=user_id, requesttype=request_type)
        send_new_request_letter_admin(lastid)
    types = get_request_types()
    requests = get_requests_by_user(current_user.user_id)
    vms = get_users_vms(current_user.username)
    for r in requests:
        try:
            date_temp = r.date_time.strftime("%d-%m-%Y")
        except:
            date_temp = r.date_time
        r.date_time = date_temp
    return render(request, 'main/make_requests.html', {'types': types, 'requests': requests, 'vms': vms})


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def manage_requests_view(request):
    requests = []
    temp_types = get_request_types()
    approved = False
    denied = False
    pending = True
    sorting_field = 'date_time'  # Default sort date - ASC
    types = {}
    for temp_type in temp_types:
        types.update({temp_type: True})
    if request.method == 'POST':
        approved = request.POST.get('approved', False)
        denied = request.POST.get('denied', False)
        pending = request.POST.get('pending', False)
        sorting_field = request.POST.get('sorting', 'date_time')
        for temp_type in types:
            types.update({temp_type: request.POST.get(temp_type.name, False)})
    if approved:
        requests += get_approved_requests().order_by(sorting_field)
    if denied:
        requests += get_denied_requests().order_by(sorting_field)
    if pending:
        requests += get_pending_requests().order_by(sorting_field)
    if not requests:
        requests += get_all_requests().order_by(sorting_field)
    temp_requests = requests.copy()
    for key, value in types.items():
        if not value:
            for temp_request in temp_requests:
                if temp_request.request_type_id.name == key.name:
                    requests.remove(temp_request)
    for r in requests:
        try:
            date_temp = r.date_time.strftime("%d-%m-%Y")
        except ValueError:
            date_temp = r.date_time
        r.date_time = date_temp
    return render(request, 'admin/manage_requests.html', {'requests': requests, 'approved': approved, 'denied': denied, 'pending': pending, 'types': types})


@login_required
def delete_vm(request):
    if request.method == 'POST':
        # Get the VM ID from the submitted form
        vm_id = request.POST['vm_id']

        try:
            proxmoxAuth.nodes(default_vm_node).qemu(vm_id).post('status/stop')  # Stop vm if its running
            time.sleep(1)  # Wait for vm to stop

            proxmoxAuth.nodes(default_vm_node).qemu(vm_id).delete()  # Delete from proxmox ve
            remove_vm_db(vm_id)  # Delete from DB
            return redirect('vms')  # Redirect to the vms page
        except Exception as e:
            return HttpResponse({'error': str(e)})

    return render(request, 'admin/delete.html')


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def view_vm(request):
    # Get sorting and search info
    sorting_field = request.session.get('sorting_admin_vm')
    keyword_search = request.session.get('search_admin_vm')
    if keyword_search is None:
        keyword_search = ""
    sync_vms_ve_and_db()

    # Get VMs from Proxmox
    vms_prox = get_list_vms_proxmox()
    # Search with a keyword if needed
    if keyword_search is not None:
        vms_sql = search_db_vm(keyword_search)
    else:
        vms_sql = get_list_db_vm()

    # Sort list of VMs
    try:
        vms_sql = vms_sql.order_by(sorting_field)
    except:
        vms_sql = vms_sql.order_by('id')  # By default sort by vm id-asc
        sorting_field = 'id'

    vm_list = []  # VMs to show
    for vm in vms_sql:
        # Check whether VM should be ignored
        if vm.id in ignore_vms_ids:
            continue

        vm_prox_status = ""
        try:
            for item in vms_prox:
                if item['vmid'] == vm.id:
                    if item['template'] == 1:
                        continue
                    vm_prox_status = item['status']
        except KeyError:
            continue

        iso = get_vm_iso(vm.id)
        if iso['name'] is not None:
            os_name = iso['name']
        else:
            os_name = "Template"

        try:
            end_date_temp = vm.end_date.strftime("%d-%m-%Y")
        except:
            end_date_temp = vm.end_date

        user_id, user_name, end_date = get_user_info_from_vmid(vm.id)

        vm_list.append({
            'status': vm_prox_status,
            'vmid': vm.id,
            'user_id': user_id,
            'user_name': user_name,
            'end_date': end_date_temp,
            'vm_name': vm.name,
            'os': os_name
        })

    # The value (second column) must be the same as field name in models
    sorting_options = {
        'VMID - ASC': 'id',
        'VMID - DSC': '-id',  # - means descending order
        'VM Name - ASC': 'name',
        'VM Name - DSC': '-name',
        'User ID - ASC': 'user_id',
        'User ID - DSC': '-user_id',
        'End date - ASC': 'end_date',
    }

    response = get_resource_consumption()
    memory_use = str(bytes_to_gb(int(response['memory']['used']))) + " Gb"
    storage_use = str(bytes_to_gb(int(response['rootfs']['used']))) + " Gb"
    cpu_usage = str("%.2f%%" % int(response['cpu']))
    render_data = {
        'vm_list': vm_list,
        'cpu_usage': cpu_usage,
        'memory_usage': memory_use,
        'storage_usage': storage_use,
        'vm_count': len(vm_list),
        'sorting_options': sorting_options,
        'sorting_field': sorting_field,
        'keyword_search': keyword_search
    }

    return render(request, 'admin/delete.html', {'info': render_data})


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def sort_vm_view_admin(request):
    # Get user input and save into a sessions variable
    sorting = request.POST['sortonadmin']
    search = request.POST['searchbarvms']
    request.session['sorting_admin_vm'] = sorting
    request.session['search_admin_vm'] = search
    return HttpResponseRedirect(reverse('vms'))


@login_required
def view_vm_per_user(request):
    # Get info of a logged in user
    user = select_user_db(request.user.username)
    # Redirect if user is admin
    if user.is_superuser == 1:
        return HttpResponseRedirect(reverse('vms'))

    vms_sql = get_users_vms(user.username)
    vms_prox = get_list_vms_proxmox()
    # Sort the list to only keep given user's vms
    vm_show_list = []
    vm_to_expire = []
    for vm in vms_sql:
        vm_prox_status = ""
        for item in vms_prox:
            if item['vmid'] == vm.id:
                vm_prox_status = item['status']
        iso = get_vm_iso(vm.id)
        if iso['name'] is not None:
            os_name = iso['name']
        else:
            os_name = "Template"
        try:
            end_date_temp = vm.end_date.strftime("%d-%m-%Y")
        except:
            end_date_temp = vm.end_date

        vm_show_list.append({
            'status': vm_prox_status,
            'vmid': vm.id,
            'user_id': vm.user_id.user_id,
            'user_name': user.first_name + " " + user.last_name,
            'end_date': end_date_temp,
            'vm_name': vm.name,
            'os': os_name

        })

        # Check if any VMs are about to expire
        if vm.end_date is not None:
            datetime_string = str(vm.end_date)
            aware_datetime = datetime.strptime(datetime_string, "%Y-%m-%d %H:%M:%S.%f%z")
            naive_datetime = aware_datetime.replace(tzinfo=timezone.utc)
            days_difference = (naive_datetime - datetime.now(timezone.utc)).days
            if days_difference <= 31:
                vm_to_expire.append(vm.id)

    render_data = {
        'vm_list': vm_show_list,
        'max_vm': user.maxAmountOfVms,
        'vm_count': len(vm_show_list),
        'name': user.first_name + " " + user.last_name + "'s",
        'expire_list': vm_to_expire
    }

    return render(request, 'main/user_vms.html', {'info': render_data})


@login_required
def extend_vm_redirect(request):
    if request.method == 'POST':
        # Get the VM ID from the submitted form
        vm_id = request.POST['vm_id']
        request.session['vm_id_session'] = vm_id  # Save vm id into session variable
        return HttpResponseRedirect(reverse('extend_vm'))


@login_required
def extend_vm(request):
    vmid = request.session.get('vm_id_session')
    render_data = {
        "vm_name": None,
        "vm_id": vmid,
        "exp_date": None,
        "user_input": ""
    }
    if vmid is not None:
        vm_model = select_vm_db(vmid)
        render_data = {
            "vm_name": vm_model.name,
            "vm_id": vmid,
            "exp_date": vm_model.end_date,
            "user_input": ""
        }
        if request.POST:
            message = request.POST['reason']
            render_data["user_input"] = message
            # Check the length of the request
            wordcount = count_words(message)
            if wordcount < 25:
                messages.info(request, "Please explain the reason for an extension using at least 25 words")
                return render(request, 'main/extend_vm.html', {'info': render_data})
            response = extend_vm_lease_time(vmid)
            response_db = insert_request_db(message, vmid, request.user.user_id, "VM Extension")
            approve_request(response_db)  # Automatically make request accepted
            send_new_request_letter_admin(response_db)
            return HttpResponseRedirect(reverse('vms_user'))
    return render(request, 'main/extend_vm.html', {'info': render_data})


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def redirect_to_user_view(request, id=None):
    if id is not None:
        redirect_url = '/admin/main/user/' + str(id) + '/change/'
        return redirect(redirect_url)
    return HttpResponseRedirect(reverse('vms'))


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def redirect_to_request_types_view(request):
    redirect_url = '/admin/main/requesttype/'
    return redirect(redirect_url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def admin_redirect(request):
    redirect_url = '/admin/main/user/'
    return redirect(redirect_url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def approve_requests(request, request_id=None):
    url = reverse(manage_requests_view)
    if request_id is None:
        return HttpResponseRedirect(url)
    approve_request(request_id)
    return HttpResponseRedirect(url)


@login_required
@user_passes_test(superuser_check, login_url=LOGIN_REDIRECT_URL)
def deny_requests(request, request_id=None):
    url = reverse(manage_requests_view)
    if request_id is None:
        return HttpResponseRedirect(url)
    deny_request(request_id)
    return HttpResponseRedirect(url)


def activate(request, uidb64, token):
    try:
        userid = force_str(urlsafe_base64_decode(uidb64))
        user = select_user_db_by_id(userid)
    except(TypeError, ValueError, OverflowError, User.DoesNotExist):
        user = None

    if user is not None and account_activation_token.check_token(user, token):
        user.email_confirmed = True
        user.save()

    send_email_confirmed_letter(user.username)
    return redirect('redirect_home')


def activateEmail(request, user, to_email):
    mail_subject = 'Activate your user account.'
    message = render_to_string('registration/template_activate_account.html', {
        'user': user.username,
        'domain': get_current_site(request).domain,
        'uid': urlsafe_base64_encode(force_bytes(user.pk)),
        'token': account_activation_token.make_token(user),
        'protocol': 'https' if request.is_secure() else 'http'
    })
    email = EmailMessage(mail_subject, message, to=[to_email])
    email.send()
